#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "heat_equation_cpu.h"

int main()
{
    //  : a <= x <= b, c <= y <= d.
    const float a = 0;
    const float b = 1;
    const float c = 2;
    const float d = 3;

    //      x  y .
    const int nx = 500;
    const int ny = 500;

    //   .
    float dx = (b - a) / (float)(nx - 1);
    float dy = (d - c) / (float)(ny - 1);

    //       .
    float dt = 1e-7f;
    int num_time_steps = 100;

    //   
    float* v0 = new float[nx * ny];
    for (int i = 0; i < nx; ++i)
    {
        for (int j = 0; j < ny; ++j)
        {
            v0[i * ny + j] = 0.0f;
        }
    }

    //    
    float* res_cpu = new float[nx * ny];
    float* res_gpu = new float[nx * ny];

    //    CPU
    heat_equation_cpu(nx, ny, dx, dy, dt, a, c, num_time_steps, v0, res_cpu);

    //    GPU
    heat_equation_cpu(nx, ny, dx, dy, dt, a, c, num_time_steps, v0, res_gpu); //TODO:  GPU 

     //     res_cpu  res_gpu.
    float difference = 0;
    for (int idx = 0; idx < nx * ny; ++idx)
        difference += (res_cpu[idx] - res_gpu[idx]) * (res_cpu[idx] - res_gpu[idx]);
    difference = sqrtf(difference);

    //  ,     .
    const float eps = 1e-5f;
    printf("Test %s\n", difference < eps ? "PASSED" : "FAILED");

    //  
    delete[] v0;
    delete[] res_cpu;
    delete[] res_gpu;
}